import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Casinomoney } from '../models/casinomoney';
import { CasinomoneyPagination } from '../models/casinomoney-pagination';
import { CasinomoneyService } from '../services/casinomoney.service';

@Injectable({
  providedIn: 'root',
})
export class CasinomoneyResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _casinomoneyService: CasinomoneyService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Casinomoney | undefined> {
    return this._casinomoneyService
      .getCasinomoneyById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CasinomoneysResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _casinomoneyService: CasinomoneyService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CasinomoneyPagination;
    casinomoneys: Casinomoney[];
  }> {
    return this._casinomoneyService.getCasinomoneys();
  }
}
