import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { CouponMoneylog } from '../models/coupon-moneylog';
import { CouponMoneylogPagination } from '../models/coupon-moneylog-pagination';
import { CouponMoneylogService } from '../services/coupon-moneylog.service';

@Injectable({
  providedIn: 'root',
})
export class CouponMoneylogResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _couponMoneylogService: CouponMoneylogService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<CouponMoneylog | undefined> {
    return this._couponMoneylogService
      .getCouponMoneylogById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CouponMoneylogsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _couponMoneylogService: CouponMoneylogService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CouponMoneylogPagination;
    couponMoneylogs: CouponMoneylog[];
  }> {
    return this._couponMoneylogService.getCouponMoneylogs();
  }
}
