import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { SessioninAdmin } from '../models/sessionin-admin';
import { SessioninAdminPagination } from '../models/sessionin-admin-pagination';
import { SessioninAdminService } from '../services/sessionin-admin.service';

@Injectable({
  providedIn: 'root',
})
export class SessioninAdminResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _sessioninAdminService: SessioninAdminService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<SessioninAdmin | undefined> {
    return this._sessioninAdminService
      .getSessioninAdminById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class SessioninAdminsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _sessioninAdminService: SessioninAdminService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: SessioninAdminPagination;
    sessioninAdmins: SessioninAdmin[];
  }> {
    return this._sessioninAdminService.getSessioninAdmins();
  }
}
