import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import {
  BehaviorSubject,
  filter,
  map,
  Observable,
  of,
  switchMap,
  take,
  tap,
  throwError,
} from 'rxjs';

import { SessioninAdmin } from '../models/sessionin-admin';
import { SessioninAdminPagination } from '../models/sessionin-admin-pagination';

@Injectable({
  providedIn: 'root',
})
export class SessioninAdminService {
  // Private
  private __pagination = new BehaviorSubject<
    SessioninAdminPagination | undefined
  >(undefined);
  private __sessioninAdmin = new BehaviorSubject<SessioninAdmin | undefined>(
    undefined
  );
  private __sessioninAdmins = new BehaviorSubject<SessioninAdmin[] | undefined>(
    undefined
  );

  /**
   * Constructor
   */
  constructor(private _httpClient: HttpClient) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  /**
   * Getter for pagination
   */
  get pagination$(): Observable<SessioninAdminPagination | undefined> {
    return this.__pagination.asObservable();
  }

  /**
   * Getter for sessioninAdmin
   */
  get sessioninAdmin$(): Observable<SessioninAdmin | undefined> {
    return this.__sessioninAdmin.asObservable();
  }

  /**
   * Getter for sessioninAdmins
   */
  get sessioninAdmins$(): Observable<SessioninAdmin[] | undefined> {
    return this.__sessioninAdmins.asObservable();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Get SessioninAdmins
   *
   *
   * @param page
   * @param size
   * @param sort
   * @param order
   * @param search
   */
  getSessioninAdmins(
    page: number = 0,
    size: number = 10,
    sort: string = 'name',
    order: 'asc' | 'desc' | '' = 'asc',
    search: string = ''
  ): Observable<{
    pagination: SessioninAdminPagination;
    sessioninAdmins: SessioninAdmin[];
  }> {
    return this._httpClient
      .get<{
        pagination: SessioninAdminPagination;
        sessioninAdmins: SessioninAdmin[];
      }>('api/apps/report/sessionin-admin/sessionin-admins', {
        params: {
          page: '' + page,
          size: '' + size,
          sort,
          order,
          search,
        },
      })
      .pipe(
        tap((response) => {
          this.__pagination.next(response.pagination);
          this.__sessioninAdmins.next(response.sessioninAdmins);
        })
      );
  }

  /**
   * Get product by id
   */
  getSessioninAdminById(id: string | null): Observable<SessioninAdmin> {
    return this.__sessioninAdmins.pipe(
      take(1),
      map((sessioninAdmins) => {
        // Find the product
        const sessioninAdmin =
          sessioninAdmins?.find((item) => item.id === id) || undefined;

        // Update the product
        this.__sessioninAdmin.next(sessioninAdmin);

        // Return the product
        return sessioninAdmin;
      }),
      switchMap((product) => {
        if (!product) {
          return throwError('Could not found product with id of ' + id + '!');
        }

        return of(product);
      })
    );
  }

  /**
   * Create product
   */
  createSessioninAdmin(): Observable<SessioninAdmin> {
    return this.sessioninAdmins$.pipe(
      take(1),
      switchMap((sessioninAdmins) =>
        this._httpClient
          .post<SessioninAdmin>('api/apps/report/sessionin-admin/product', {})
          .pipe(
            map((newSessioninAdmin) => {
              // Update the sessioninAdmins with the new product
              if (!!sessioninAdmins) {
                this.__sessioninAdmins.next([
                  newSessioninAdmin,
                  ...sessioninAdmins,
                ]);
              }

              // Return the new product
              return newSessioninAdmin;
            })
          )
      )
    );
  }
}
