import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { SessioninInfo } from '../models/sessionin-info';
import { SessioninInfoPagination } from '../models/sessionin-info-pagination';
import { SessioninInfoService } from '../services/sessionin-info.service';

@Injectable({
  providedIn: 'root',
})
export class SessioninInfoResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _sessioninInfoService: SessioninInfoService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<SessioninInfo | undefined> {
    return this._sessioninInfoService
      .getSessioninInfoById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class SessioninInfosResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _sessioninInfoService: SessioninInfoService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: SessioninInfoPagination;
    sessioninInfos: SessioninInfo[];
  }> {
    return this._sessioninInfoService.getSessioninInfos();
  }
}
