import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { SessioninOverlap } from '../models/sessionin-overlap';
import { SessioninOverlapPagination } from '../models/sessionin-Overlap-pagination';
import { SessioninOverlapService } from '../services/sessionin-overlap.service';

@Injectable({
  providedIn: 'root',
})
export class SessioninOverlapResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _sessioninOverlapService: SessioninOverlapService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<SessioninOverlap | undefined> {
    return this._sessioninOverlapService
      .getSessioninOverlapById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class SessioninOverlapsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _sessioninOverlapService: SessioninOverlapService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: SessioninOverlapPagination;
    sessioninOverlaps: SessioninOverlap[];
  }> {
    return this._sessioninOverlapService.getSessioninOverlaps();
  }
}
