import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import {
  BehaviorSubject,
  filter,
  map,
  Observable,
  of,
  switchMap,
  take,
  tap,
  throwError,
} from 'rxjs';

import { SessioninOverlap } from '../models/sessionin-overlap';
import { SessioninOverlapPagination } from '../models/sessionin-Overlap-pagination';

@Injectable({
  providedIn: 'root',
})
export class SessioninOverlapService {
  // Private
  private __pagination = new BehaviorSubject<
    SessioninOverlapPagination | undefined
  >(undefined);
  private __sessioninOverlap = new BehaviorSubject<
    SessioninOverlap | undefined
  >(undefined);
  private __sessioninOverlaps = new BehaviorSubject<
    SessioninOverlap[] | undefined
  >(undefined);

  /**
   * Constructor
   */
  constructor(private _httpClient: HttpClient) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  /**
   * Getter for pagination
   */
  get pagination$(): Observable<SessioninOverlapPagination | undefined> {
    return this.__pagination.asObservable();
  }

  /**
   * Getter for sessioninOverlap
   */
  get sessioninOverlap$(): Observable<SessioninOverlap | undefined> {
    return this.__sessioninOverlap.asObservable();
  }

  /**
   * Getter for sessioninOverlaps
   */
  get sessioninOverlaps$(): Observable<SessioninOverlap[] | undefined> {
    return this.__sessioninOverlaps.asObservable();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Get SessioninOverlaps
   *
   *
   * @param page
   * @param size
   * @param sort
   * @param order
   * @param search
   */
  getSessioninOverlaps(
    page: number = 0,
    size: number = 10,
    sort: string = 'name',
    order: 'asc' | 'desc' | '' = 'asc',
    search: string = ''
  ): Observable<{
    pagination: SessioninOverlapPagination;
    sessioninOverlaps: SessioninOverlap[];
  }> {
    return this._httpClient
      .get<{
        pagination: SessioninOverlapPagination;
        sessioninOverlaps: SessioninOverlap[];
      }>('api/apps/report/sessionin-overlap/sessionin-overlaps', {
        params: {
          page: '' + page,
          size: '' + size,
          sort,
          order,
          search,
        },
      })
      .pipe(
        tap((response) => {
          this.__pagination.next(response.pagination);
          this.__sessioninOverlaps.next(response.sessioninOverlaps);
        })
      );
  }

  /**
   * Get product by id
   */
  getSessioninOverlapById(id: string | null): Observable<SessioninOverlap> {
    return this.__sessioninOverlaps.pipe(
      take(1),
      map((sessioninOverlaps) => {
        // Find the product
        const sessioninOverlap =
          sessioninOverlaps?.find((item) => item.id === id) || undefined;

        // Update the product
        this.__sessioninOverlap.next(sessioninOverlap);

        // Return the product
        return sessioninOverlap;
      }),
      switchMap((product) => {
        if (!product) {
          return throwError('Could not found product with id of ' + id + '!');
        }

        return of(product);
      })
    );
  }

  /**
   * Create product
   */
  createSessioninOverlap(): Observable<SessioninOverlap> {
    return this.sessioninOverlaps$.pipe(
      take(1),
      switchMap((sessioninOverlaps) =>
        this._httpClient
          .post<SessioninOverlap>(
            'api/apps/report/sessionin-overlap/product',
            {}
          )
          .pipe(
            map((newSessioninOverlap) => {
              // Update the sessioninOverlaps with the new product
              if (!!sessioninOverlaps) {
                this.__sessioninOverlaps.next([
                  newSessioninOverlap,
                  ...sessioninOverlaps,
                ]);
              }

              // Return the new product
              return newSessioninOverlap;
            })
          )
      )
    );
  }
}
