import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Statistics } from '../models/statistics';
import { StatisticsPagination } from '../models/statistics-pagination';
import { StatisticsService } from '../services/statistics.service';

@Injectable({
  providedIn: 'root',
})
export class StatisticsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _statisticsService: StatisticsService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Statistics | undefined> {
    return this._statisticsService
      .getStatisticsById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class StatisticssResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _statisticsService: StatisticsService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: StatisticsPagination;
    statisticss: Statistics[];
  }> {
    return this._statisticsService.getStatisticss();
  }
}
